import random
from typing import List

from daaja.augmentors.sentence.sentence_augmentor import SentenceAugmentor
from daaja.tokenizer import Tokenizer


class RandamSwapAugmentor(SentenceAugmentor):
    def __init__(self,
                 alpha: float = 0.1,
                 tokenizer: Tokenizer = Tokenizer()) -> None:
        self.alpha = alpha
        self.tokenizer = tokenizer

    def augment(self, sentence: str) -> str:
        """Randomly swap two words in the sentence n times

        Args:
            sentence (str): sentence

        Returns:
            str: swaped sentence
        """
        tokens, _ = self.tokenizer.tokenize(sentence)
        n = max(1, int(self.alpha * len(tokens)))

        converted_tokens = tokens.copy()
        for _ in range(n):
            converted_tokens = self._swap_word(converted_tokens)
        return "".join(converted_tokens)

    def _swap_word(self, tokens: List[str]) -> List[str]:
        random_idx_1 = random.randint(0, len(tokens) - 1)
        random_idx_2 = random_idx_1
        cnt = 0
        while random_idx_1 == random_idx_2:
            random_idx_2 = random.randint(0, len(tokens) - 1)
            cnt += 1
            if cnt > 3:
                return tokens
        tokens[random_idx_1], tokens[random_idx_2] = tokens[random_idx_2], tokens[random_idx_1]
        return tokens



from daaja.augmentors.sentence.back_translation_augmentor import \
    BackTranslationAugmentor
from daaja.augmentors.sentence.contextual_augmentor import ContextualAugmentor
from daaja.augmentors.sentence.randam_delete_augmentor import \
    RandamDeleteAugmentor
from daaja.augmentors.sentence.randam_insert_augmentor import \
    RandamInsertAugmentor
from daaja.augmentors.sentence.randam_swap_augmentor import RandamSwapAugmentor
from daaja.augmentors.sentence.synonym_replace_augmentor import \
    SynonymReplaceAugmentor



import torch
from daaja.augmentors.sentence.sentence_augmentor import SentenceAugmentor
from transformers import AutoModelForSeq2SeqLM, AutoTokenizer, pipeline


class BackTranslationAugmentor(SentenceAugmentor):
    def __init__(self,
                 from_model_name: str = "Helsinki-NLP/opus-mt-ja-en",
                 to_model_name: str = "Helsinki-NLP/opus-mt-en-jap",
                 device: int = -1) -> None:
        # ja -> en
        self.from_pipeline = pipeline("translation",
                                      model=AutoModelForSeq2SeqLM.from_pretrained(from_model_name),
                                      tokenizer=AutoTokenizer.from_pretrained(from_model_name),
                                      device=device)
        # en -> ja
        self.to_pipeline = pipeline("translation",
                                    model=AutoModelForSeq2SeqLM.from_pretrained(to_model_name),
                                    tokenizer=AutoTokenizer.from_pretrained(to_model_name),
                                    device=device)

    @torch.no_grad()
    def augment(self, sentence: str) -> str:
        """Back translation Augmentor

        Args:
            sentence (str): sentence

        Returns:
            str: Back translation sentence
        """
        to_text = self.from_pipeline(sentence)[0]["translation_text"]
        from_text = self.to_pipeline(to_text)[0]["translation_text"]
        return from_text



from typing import List


class SequenceLabelingAugmentor:
    def __init__(self) -> None:
        pass

    def augment(self,
                tokens: List[str],
                labels: List[str]) -> str:
        raise NotImplementedError



from typing import List, Tuple

import numpy as np
from daaja.augmentors.sequence_labeling.sequence_labeling_augmentor import \
    SequenceLabelingAugmentor
from daaja.augmentors.sequence_labeling.utils import EntityDict


class MentionReplacementAugmentor(SequenceLabelingAugmentor):
    def __init__(self,
                 entity_dict: EntityDict,
                 p: float = 0.1) -> None:
        self.p = p
        self.entity_dict = entity_dict

    def augment(self,
                tokens: List[str],
                labels: List[str]) -> Tuple[List[str], List[str]]:
        generated_tokens, generated_labels = [], []
        for i, (label, token) in enumerate(zip(labels, tokens)):
            if label == "O":
                generated_tokens_i = [token]
                generated_labels_i = [label]
            elif label.startswith("B"):
                label_type = label[2:]
                is_change_mention = np.random.binomial(1, self.p, 1)[0]
                if is_change_mention:
                    candidates = self.entity_dict[label_type]
                    random_idx = np.random.choice(len(candidates), 1)[0]
                    replaced_mention = candidates[random_idx].split()
                    generated_tokens_i = replaced_mention
                    generated_labels_i = ["B-" + label_type if i == 0 else "I-" + label_type for i in range(len(replaced_mention))]
                else:
                    generated_tokens_i = [token]
                    generated_labels_i = [label]
                    next_idx = i + 1
                    while next_idx < len(tokens) and tokens[next_idx].startswith("I"):
                        generated_tokens_i.append(tokens[next_idx])
                        generated_labels_i.append(labels[next_idx])
                        next_idx += 1
            elif label[0] == "I":
                continue
            else:
                raise ValueError("unreachable line...")
            generated_tokens.extend(generated_tokens_i)
            generated_labels.extend(generated_labels_i)
        return generated_tokens, generated_labels



import random
from typing import List, Optional

from daaja.augmentors.sentence.sentence_augmentor import SentenceAugmentor
from daaja.resouces import Resouces
from daaja.tokenizer import Tokenizer


class RandamInsertAugmentor(SentenceAugmentor):
    def __init__(self,
                 alpha: float = 0.1,
                 resouces: Resouces = Resouces(),
                 tokenizer: Tokenizer = Tokenizer()) -> None:
        self.alpha = alpha
        self.resouces = resouces
        self.tokenizer = tokenizer

    def augment(self, sentence: str) -> str:
        """Randomly insert a selected token synonym from selected_tokens in sentence.

        Args:
            sentence (str): sentence

        Returns:
            str: swaped sentence
        """
        tokens, selected_tokens = self.tokenizer.tokenize(sentence)
        n = max(1, int(self.alpha * len(tokens)))

        converted_tokens = tokens.copy()
        for _ in range(n):
            converted_tokens = self._add_word(converted_tokens, selected_tokens)
        return "".join(converted_tokens)

    def _add_word(self, tokens: List[str], selected_tokens: List[Optional[str]]) -> List[str]:
        synonyms = []
        selected_not_none_tokens: List[str] = [token for token in selected_tokens if token is not None]
        cnt = 0
        while len(synonyms) < 1:
            random_selected_token = selected_not_none_tokens[random.randint(0, len(selected_not_none_tokens) - 1)]
            synonyms = self.resouces.get_synonyms(random_selected_token)
            cnt += 1
            # NOTE: 10 is magic number.
            # If this function don't see a synonym after looking for it 10 times, done.
            if cnt >= 10:
                return tokens
        random_idx = random.randint(0, len(tokens) - 1)
        random_selected_synonym = synonyms[0]
        tokens.insert(random_idx, random_selected_synonym)
        return tokens



from typing import List, Tuple

import numpy as np
from daaja.augmentors.sequence_labeling.sequence_labeling_augmentor import \
    SequenceLabelingAugmentor
from daaja.augmentors.sequence_labeling.utils import TokenAndProbInLabel
from daaja.resouces import Resouces


class LabelwiseTokenReplacementAugmentor(SequenceLabelingAugmentor):
    def __init__(self,
                 token_and_prob_in_label: TokenAndProbInLabel,
                 p: float = 0.1,
                 resouces: Resouces = Resouces()) -> None:
        self.p = p
        self.resouces = resouces
        self.token_and_prob_in_label = token_and_prob_in_label

    def augment(self,
                tokens: List[str],
                labels: List[str]) -> Tuple[List[str], List[str]]:
        masks = np.random.binomial(1, self.p, len(tokens))
        generated_tokens = []
        for mask, token, label in zip(masks, tokens, labels):
            if mask == 0 or self.resouces.is_stopword(token):
                generated_token = token
            else:
                random_idx = np.random.choice(len(self.token_and_prob_in_label[label][1]), 1, p=self.token_and_prob_in_label[label][1])[0]
                generated_token = self.token_and_prob_in_label[label][0][random_idx]
            generated_tokens.append(generated_token)

        return generated_tokens, labels



from typing import List, Optional, Tuple

from sudachipy import dictionary as sudachi_dictionary
from sudachipy import tokenizer as sudachi_tokenizer


class Tokenizer:
    def __init__(self, sudachi_mode: str = "C") -> None:
        self.tokenizer_obj = sudachi_dictionary.Dictionary().create()
        if sudachi_mode == "A":
            self.mode = sudachi_tokenizer.Tokenizer.SplitMode.A
        elif sudachi_mode == "B":
            self.mode = sudachi_tokenizer.Tokenizer.SplitMode.B
        elif sudachi_mode == "C":
            self.mode = sudachi_tokenizer.Tokenizer.SplitMode.C
        else:
            raise ValueError("Select sudachi_mode from A, B, and C.")

    def tokenize(self,
                 text: str,
                 stopwords: List[str] = [],
                 synonym_target_pos: List[str] = ["名詞", "動詞"]) -> Tuple[List[str], List[Optional[str]]]:
        m = self.tokenizer_obj.tokenize(text, self.mode)

        tokens: List[str] = [mi.surface() for mi in m]
        cleaned_tokens: List[Optional[str]] = [mi.dictionary_form() if mi.part_of_speech()[0] in synonym_target_pos else None for mi in m]
        # Tokens in which the part of speech and stopwords not in synonym_target_pos are set to None.
        selected_tokens: List[Optional[str]] = [token if (token is not None) and (token not in stopwords) else None for token in cleaned_tokens]

        return tokens, selected_tokens


if __name__ == "__main__":
    toeknizer = Tokenizer()
    toeknizer.tokenize("類似するデータを生成する記事を書いてます。", ["類似"])



from daaja.augmentors.sequence_labeling.labelwise_token_replacement_augmentor import \
    LabelwiseTokenReplacementAugmentor
from daaja.augmentors.sequence_labeling.mention_replacement_augmentor import \
    MentionReplacementAugmentor
from daaja.augmentors.sequence_labeling.shuffle_within_segments_augmentor import \
    ShuffleWithinSegmentsAugmentor
from daaja.augmentors.sequence_labeling.synonym_replacement_augmentor import \
    SynonymReplacementAugmentor



import random
from typing import List, Tuple

from daaja.augmentors.sequence_labeling.sequence_labeling_augmentor import \
    SequenceLabelingAugmentor
from tqdm import tqdm


class SequentialSequenceLabelingFlow:
    def __init__(self,
                 augmentors: List[SequenceLabelingAugmentor],
                 num_aug: int) -> None:
        self.augmentors = augmentors
        self.num_aug = num_aug

    def augments(self, tokens: List[str], labels: List[str]) -> Tuple[List[List[str]], List[List[str]]]:
        augmented_tokens_and_labels = []
        num_per_technique = int(self.num_aug / len(self.augmentors)) + 1

        for augmentor in tqdm(self.augmentors, desc="augment"):
            for _ in range(num_per_technique):
                aug_tokens, aug_labels = augmentor.augment(tokens, labels)
                augmented_tokens_and_labels.append([aug_tokens, aug_labels])

        random.shuffle(augmented_tokens_and_labels)

        if self.num_aug >= 1:
            augmented_tokens_and_labels = augmented_tokens_and_labels[:self.num_aug]
        else:
            keep_prob = self.num_aug / len(augmented_tokens_and_labels)
            augmented_tokens_and_labels = [s for s in augmented_tokens_and_labels if random.uniform(0, 1) < keep_prob]

        # Append the original sentence
        augmented_tokens_and_labels.append([tokens, labels])
        augmented_tokens_list = [tokens_list for tokens_list, _ in augmented_tokens_and_labels]
        augmented_labels_list = [labels_list for _, labels_list in augmented_tokens_and_labels]
        return augmented_tokens_list, augmented_labels_list



import random

from daaja.augmentors.sentence.sentence_augmentor import SentenceAugmentor
from daaja.tokenizer import Tokenizer


class RandamDeleteAugmentor(SentenceAugmentor):
    def __init__(self,
                 p: float = 0.05,
                 tokenizer: Tokenizer = Tokenizer()) -> None:
        self.p = p
        self.tokenizer = tokenizer

    def augment(self, sentence: str) -> str:
        tokens, _ = self.tokenizer.tokenize(sentence)
        if len(tokens) == 1:
            return sentence

        converted_tokens = []
        for token in tokens:
            r = random.uniform(0, 1)
            if r < self.p:
                continue
            converted_tokens.append(token)

        # None of the tokens exist
        if len(converted_tokens) == 0:
            random_idx = random.randint(0, len(tokens) - 1)
            return "".join(tokens[random_idx])

        return "".join(converted_tokens)



import gzip
import io
import logging
import os
import shutil
import sqlite3
from pathlib import Path
from typing import List

import pandas as pd
import requests

logger = logging.getLogger(__name__)


class Resouces:
    DIR = Path("/tmp/eda_ja")
    WORDNET_PATH = DIR / Path("wnjpn.db.gz")
    WORDNER_URL = "http://compling.hss.ntu.edu.sg/wnja/data/1.1/wnjpn.db.gz"
    STOPWORDS_PATH = DIR / Path("stopwords.txt")
    STOPWORDS_URL = "http://svn.sourceforge.jp/svnroot/slothlib/CSharp/Version1/SlothLib/NLP/Filter/StopWord/word/Japanese.txt"

    def __init__(self) -> None:
        os.makedirs(self.DIR, exist_ok=True)

        # Download and Load wordnet
        self.download_wordnet()
        connect = sqlite3.connect(self.WORDNET_PATH)
        query = 'SELECT synset, lemma FROM sense, word USING (wordid) WHERE sense.lang="jpn"'
        self.sence_words = pd.read_sql(query, connect)

        #  Download and Load stopwords
        self.download_stopwords()
        with open(self.STOPWORDS_PATH) as f:
            stopwords_txt = f.read()
        self.stopwords = [line.strip() for line in stopwords_txt.split("\n") if len(line.strip()) > 0]

    def download_stopwords(self) -> None:
        if os.path.isfile(self.STOPWORDS_PATH):
            logging.info("The stopword file already exists.")
            return
        res = requests.get(self.STOPWORDS_URL).content.decode("utf-8")
        with open(self.STOPWORDS_PATH, "w") as f:
            f.write(res)

    def download_wordnet(self) -> None:
        if os.path.isfile(self.WORDNET_PATH):
            logging.info("The wordnet file already exists.")
            return

        logging.info("Downloading wordnet.")
        with gzip.open(io.BytesIO(requests.get(self.WORDNER_URL).content), "rb") as f_gz:
            with open(self.WORDNET_PATH, "wb") as f:
                shutil.copyfileobj(f_gz, f)

    def get_synonyms(self, word: str) -> List[str]:
        synsets = self.sence_words.loc[self.sence_words.lemma == word, "synset"]
        synset_words = set(self.sence_words.loc[self.sence_words.synset.isin(synsets), "lemma"])

        if word in synset_words:
            synset_words.remove(word)
        return list(synset_words)

    def is_stopword(self, word: str) -> bool:
        return word in self.stopwords


if __name__ == "__main__":
    resources = Resouces()
    resources.get_synonyms("データ")



import random

from daaja.augmentors.sentence.sentence_augmentor import SentenceAugmentor
from daaja.resouces import Resouces
from daaja.tokenizer import Tokenizer


class SynonymReplaceAugmentor(SentenceAugmentor):
    def __init__(self,
                 alpha: float = 0.1,
                 resouces: Resouces = Resouces(),
                 tokenizer: Tokenizer = Tokenizer()) -> None:
        self.alpha = alpha
        self.resouces = resouces
        self.tokenizer = tokenizer

    def augment(self, sentence: str) -> str:
        """Randomly replace n tokens with their synonyms.

        Args:
            sentence (str): sentence

        Returns:
            str: synonym_replaced_sentence
        """
        tokens, selected_tokens = self.tokenizer.tokenize(sentence)
        n = max(1, int(self.alpha * len(tokens)))

        converted_tokens = tokens.copy()
        selected_token_idxes = [i for i, token in enumerate(selected_tokens) if token is not None]
        random.shuffle(selected_token_idxes)
        cnt = 0
        for idx in selected_token_idxes:
            token = tokens[idx]
            selected_token = selected_tokens[idx]
            synonyms = self.resouces.get_synonyms(selected_token)
            if len(synonyms) == 0:
                continue
            synonym = random.choice(synonyms)
            converted_tokens = [synonym if converted_token == token else converted_token for converted_token in converted_tokens]
            cnt += 1
            if cnt >= n:
                break
        return "".join(converted_tokens)



import random
from typing import List

from daaja.augmentors.sentence.sentence_augmentor import SentenceAugmentor
from tqdm import tqdm


class SequentialSentenceFlow:
    def __init__(self,
                 augmentors: List[SentenceAugmentor],
                 num_aug: int) -> None:
        self.augmentors = augmentors
        self.num_aug = num_aug

    def augments(self, sentence: str) -> List[str]:
        augmented_sentences = []
        num_per_technique = int(self.num_aug / len(self.augmentors)) + 1

        for augmentor in tqdm(self.augmentors, desc="augment"):
            for _ in range(num_per_technique):
                augmented_sentences.append(augmentor.augment(sentence))

        random.shuffle(augmented_sentences)

        if self.num_aug >= 1:
            augmented_sentences = augmented_sentences[:self.num_aug]
        else:
            keep_prob = self.num_aug / len(augmented_sentences)
            augmented_sentences = [s for s in augmented_sentences if random.uniform(0, 1) < keep_prob]

        # Append the original sentence
        augmented_sentences.append(sentence)
        return augmented_sentences



from typing import List

from daaja.augmentors.sentence.randam_delete_augmentor import \
    RandamDeleteAugmentor
from daaja.augmentors.sentence.randam_insert_augmentor import \
    RandamInsertAugmentor
from daaja.augmentors.sentence.randam_swap_augmentor import RandamSwapAugmentor
from daaja.augmentors.sentence.synonym_replace_augmentor import \
    SynonymReplaceAugmentor
from daaja.flows.sequential_flow import SequentialSentenceFlow
from daaja.resouces import Resouces
from daaja.tokenizer import Tokenizer


class EasyDataAugmentor:
    def __init__(self, alpha_sr: float, alpha_ri: float, alpha_rs: float, p_rd: float, num_aug: int):
        resouces = Resouces()
        self.tokenizer = Tokenizer()

        self.flow = SequentialSentenceFlow(
            augmentors=[
                SynonymReplaceAugmentor(alpha=alpha_sr, resouces=resouces, tokenizer=self.tokenizer),
                RandamInsertAugmentor(alpha=alpha_ri, resouces=resouces, tokenizer=self.tokenizer),
                RandamSwapAugmentor(alpha=alpha_rs, tokenizer=self.tokenizer),
                RandamDeleteAugmentor(p=p_rd, tokenizer=self.tokenizer)],
            num_aug=num_aug)

    def augments(self, sentence: str) -> List[str]:
        return self.flow.augments(sentence)



import argparse
import csv
from dataclasses import dataclass
from pathlib import Path
from typing import List

from daaja.methods.eda.easy_data_augmentor import EasyDataAugmentor
from tqdm import tqdm


@dataclass
class Data:
    label: str
    sentence: str

    def print_l(self):
        return "{}\t{}".format(self.label, self.sentence)


def main(input_path: Path,
         output_path: Path,
         alpha_sr: float,
         alpha_ri: float,
         alpha_rs: float,
         p_rd: float,
         num_aug: int):
    datum = load_tsv(input_path=input_path)
    eda = EasyDataAugmentor(alpha_sr=alpha_sr, alpha_ri=alpha_ri, alpha_rs=alpha_rs, p_rd=p_rd, num_aug=num_aug)

    results = []
    for data in tqdm(datum):
        aug_sentences = eda.augments(data.sentence)
        results.extend([Data(data.label, aug_sentence) for aug_sentence in aug_sentences])

    write_tsv(output_path=output_path, datum=results)


def load_tsv(input_path: Path) -> List[Data]:
    results = []
    with open(input_path, encoding="utf-8", newline='') as f:
        for cols in csv.reader(f, delimiter='\t'):
            results.append(Data(cols[0], cols[1]))
    return results


def write_tsv(output_path: Path, datum: List[Data]):
    with open(output_path, mode="w") as f:
        f.write('\n'.join([data.print_l() for data in datum]))


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    # file
    parser.add_argument("--input", required=True, type=Path)
    parser.add_argument("--output", required=True, type=Path)
    # params
    parser.add_argument("--num_aug", default=9, type=int)
    # alpha
    parser.add_argument("--alpha_sr", default=0.1, type=float,
                        help="percent of words in each sentence to be replaced by synonyms")
    parser.add_argument("--alpha_rd", default=0.1, type=float,
                        help="percent of words in each sentence to be deleted")
    parser.add_argument("--alpha_ri", default=0.1, type=float,
                        help="percent of words in each sentence to be inserted")
    parser.add_argument("--alpha_rs", default=0.1, type=float,
                        help="percent of words in each sentence to be swapped")
    args = parser.parse_args()

    main(args.input, args.output, args.alpha_sr, args.alpha_ri, args.alpha_rs, args.alpha_rd, args.num_aug)



import random

import torch
from daaja.augmentors.sentence.sentence_augmentor import SentenceAugmentor
from transformers import pipeline


class ContextualAugmentor(SentenceAugmentor):
    def __init__(self,
                 p: float = 0.15,
                 max_predictions_per_seq: int = 100000,
                 model_name: str = "cl-tohoku/bert-base-japanese-whole-word-masking",
                 device: int = -1) -> None:
        self.p = p
        self.max_predictions_per_seq = max_predictions_per_seq
        self.pipeline = pipeline("fill-mask",
                                 model=model_name,
                                 top_k=2,
                                 device=device)
        self.mask_token = self.pipeline.tokenizer.mask_token

    @torch.no_grad()
    def augment(self, sentence: str) -> str:
        """Contextual Augmentor

        Args:
            sentence (str): sentence

        Returns:
            str: Contextual augmented sentence
        """
        tokens = self.pipeline.tokenizer.tokenize(sentence)
        n = min(self.max_predictions_per_seq, max(1, int(self.p * len(tokens))))
        random_idx = list(range(len(tokens)))
        random.shuffle(random_idx)

        converted_tokens = tokens.copy()
        for random_i in random_idx[:n]:
            converted_tokens[random_i] = self.mask_token
            aug_texts = self.pipeline("".join(converted_tokens))
            for aug_text in aug_texts:
                token_str = aug_text["token_str"].replace(" ", "")
                if token_str == tokens[random_i]:
                    continue
                converted_tokens[random_i] = token_str
                break

        return "".join(converted_tokens)



import random
from typing import List, Tuple

import numpy as np
from daaja.augmentors.sequence_labeling.sequence_labeling_augmentor import \
    SequenceLabelingAugmentor


class ShuffleWithinSegmentsAugmentor(SequenceLabelingAugmentor):
    def __init__(self,
                 p: float = 0.1) -> None:
        self.p = p

    def augment(self,
                tokens: List[str],
                labels: List[str]) -> Tuple[List[str], List[str]]:
        generated_tokens, generated_labels = [], []
        shuffled_idx = self._shuffle_within_segments(labels)
        assert len(shuffled_idx) == len(labels)
        for i in shuffled_idx:
            generated_token = tokens[i]
            generated_label = labels[i]
            generated_tokens.append(generated_token)
            generated_labels.append(generated_label)
        return generated_tokens, generated_labels

    def _shuffle_within_segments(self, labels: List[str]) -> List[int]:
        segments = [0]
        for i, label in enumerate(labels):
            if i == 0:
                continue
            if label == "O":
                if labels[i - 1] == "O":
                    segments.append(segments[-1])
                else:
                    segments.append(segments[-1] + 1)
            elif label.startswith("B"):
                segments.append(segments[-1] + 1)
            else:
                segments.append(segments[-1])

        shuffled_idx = []
        start, end = 0, 0
        while start < len(segments) and end < len(segments):
            while end < len(segments) and segments[end] == segments[start]:
                end += 1
            segment = [i for i in range(start, end)]
            if len(segment) > 1 and np.random.binomial(1, self.p, 1)[0] == 1:
                random.shuffle(segment)
            shuffled_idx += segment
            start = end
        return shuffled_idx



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['daaja',
 'daaja.augmentors',
 'daaja.augmentors.sentence',
 'daaja.augmentors.sequence_labeling',
 'daaja.flows',
 'daaja.methods.eda',
 'daaja.methods.ner_sda']

package_data = \
{'': ['*']}

install_requires = \
['SudachiDict-core>=20211220,<20211221',
 'SudachiPy>=0.6.3,<0.7.0',
 'fugashi>=1.1.2,<2.0.0',
 'ipadic>=1.0.0,<2.0.0',
 'numpy>=1.3',
 'pandas>=1.2',
 'requests>=2.27.1,<3.0.0',
 'sentencepiece>=0.1.96,<0.2.0',
 'torch>=1.10.2,<2.0.0',
 'tqdm>=4.62.3,<5.0.0',
 'transformers>=4.16.2,<5.0.0']

setup_kwargs = {
    'name': 'daaja',
    'version': '0.0.5',
    'description': 'NLP data augmentation tool',
    'long_description': 'None',
    'author': 'Koga Kobayashi',
    'author_email': 'kajyuuen@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7.1,<4.0',
}


setup(**setup_kwargs)



from typing import List, Tuple

from daaja.augmentors.sequence_labeling.labelwise_token_replacement_augmentor import \
    LabelwiseTokenReplacementAugmentor
from daaja.augmentors.sequence_labeling.mention_replacement_augmentor import \
    MentionReplacementAugmentor
from daaja.augmentors.sequence_labeling.shuffle_within_segments_augmentor import \
    ShuffleWithinSegmentsAugmentor
from daaja.augmentors.sequence_labeling.synonym_replacement_augmentor import \
    SynonymReplacementAugmentor
from daaja.augmentors.sequence_labeling.utils import (get_entity_dict,
                                                      get_token2prob_in_label)
from daaja.flows.sequential_sequenceLabeling_flow import \
    SequentialSequenceLabelingFlow
from daaja.resouces import Resouces


class SimpleDataAugmentationforNER:
    def __init__(self,
                 tokens_list: List[List[str]],
                 labels_list: List[List[str]],
                 p_power: float,
                 p_lwtr: float,
                 p_mr: float,
                 p_sis: float,
                 p_sr: float,
                 num_aug: int):
        resouces = Resouces()
        entity_dict = get_entity_dict(tokens_list, labels_list)
        token_and_prob_in_label = get_token2prob_in_label(tokens_list, labels_list, p_power=p_power)

        self.flow = SequentialSequenceLabelingFlow(
            augmentors=[
                LabelwiseTokenReplacementAugmentor(token_and_prob_in_label, p=p_lwtr, resouces=resouces),
                MentionReplacementAugmentor(entity_dict=entity_dict, p=p_mr),
                ShuffleWithinSegmentsAugmentor(p=p_sis),
                SynonymReplacementAugmentor(p=p_sr, resouces=resouces)
            ],
            num_aug=num_aug
        )

    def augments(self, tokens: List[str], labels: List[str]) -> Tuple[List[List[str]], List[List[str]]]:
        return self.flow.augments(tokens, labels)



from collections import defaultdict
from typing import DefaultDict, Dict, List, Tuple

import numpy as np

EntityDict = Dict[str, List[str]]
TokenAndProbInLabel = DefaultDict[str, Tuple[List[str], List[float]]]


def get_entity_dict(tokens_list: List[List[str]],
                    labels_list: List[List[str]]) -> EntityDict:
    splitter = " "
    entities: DefaultDict[str, List[str]] = defaultdict(list)
    for tokens, labels in zip(tokens_list, labels_list):
        entity = []
        current_label = None
        for token, label in zip(tokens, labels):
            if label == "O":
                entities["O"].append(token)
                if current_label is not None:
                    entities[current_label].append(f"{splitter}".join(entity))
                entity = []
                current_label = None
            if label[0].startswith("B"):
                current_label = label[2:]
                entity.append(token)
            elif label[0].startswith("I"):
                entity.append(token)
        if current_label is not None:
            entities[current_label].append(f"{splitter}".join(entity))
    entity_dict = {k: list(set(v)) for k, v in entities.items() for vi in v}
    return entity_dict


def get_token2prob_in_label(tokens_list: List[List[str]],
                            labels_list: List[List[str]],
                            p_power: float = 1) -> TokenAndProbInLabel:
    counter: DefaultDict[str, DefaultDict[str, int]] = defaultdict(lambda: defaultdict(int))
    for tokens, labels in zip(tokens_list, labels_list):
        for token, label in zip(tokens, labels):
            counter[label][token] += 1

    token_and_prob_in_label: DefaultDict[str, Tuple[List[str], List[float]]] = defaultdict(tuple)
    for label, counter_i in counter.items():
        sum_n = sum(counter_i.values())
        tokens, probs = [], []
        for token, cnt in counter_i.items():
            prob = np.power(cnt, p_power) / sum_n
            tokens.append(token)
            probs.append(prob)
        token_and_prob_in_label[label] = (tokens, probs)
    return token_and_prob_in_label



import argparse
from dataclasses import dataclass
from pathlib import Path
from typing import List, Tuple

from daaja.methods.ner_sda.simple_data_augmentation_for_ner import \
    SimpleDataAugmentationforNER
from tqdm import tqdm


@dataclass
class Data:
    label: str
    sentence: str

    def print_l(self):
        return "{}\t{}".format(self.label, self.sentence)


def main(input_path: Path,
         output_path: Path,
         p_power: float,
         p_lwtr: float,
         p_mr: float,
         p_sis: float,
         p_sr: float,
         num_aug: int):
    tokens_list, labels_list = load_conll(input_path=input_path)
    sda = SimpleDataAugmentationforNER(tokens_list=tokens_list, labels_list=labels_list, p_power=p_power,
                                       p_lwtr=p_lwtr, p_mr=p_mr, p_sis=p_sis, p_sr=p_sr, num_aug=num_aug)

    result_tokens_list, result_labels_list = [], []
    for tokens, labels in tqdm(zip(tokens_list, labels_list)):
        arg_tokens_list, arg_labels_list = sda.augments(tokens, labels)
        result_tokens_list.extend(arg_tokens_list)
        result_labels_list.extend(arg_labels_list)

    write_conll(output_path=output_path, tokens_list=result_tokens_list, labels_list=result_labels_list)


def load_conll(input_path: Path) -> Tuple[List[List[str]], List[List[str]]]:
    tokens_list, labels_list = [], []
    tokens, labels = [], []
    for line in input_path.read_text().split("\n"):
        cols = line.split("\t")
        if len(cols) != 2:
            if len(tokens) > 0:
                tokens_list.append(tokens)
                labels_list.append(labels)
                tokens, labels = [], []
            continue
        tokens.append(cols[0])
        labels.append(cols[1])
    if len(tokens) > 0:
        tokens_list.append(tokens)
        labels_list.append(labels)
    return tokens_list, labels_list


def write_conll(output_path: Path,
                tokens_list: List[List[str]],
                labels_list: List[List[str]]):
    text = ""
    for tokens, labels in zip(tokens_list, labels_list):
        for token, label in zip(tokens, labels):
            text += "{}\t{}\n".format(token, label)
        text += "\n"

    with open(output_path, mode="w") as f:
        f.write(text)


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    # file
    parser.add_argument("--input", required=True, type=Path)
    parser.add_argument("--output", required=True, type=Path)
    # params
    parser.add_argument("--num_aug", default=4, type=int)
    # alpha
    parser.add_argument("--p_power", default=1, type=float,
                        help="p_power hyper params")
    parser.add_argument("--p_lwtr", default=0.1, type=float,
                        help="percent of words in each sentence to be labelwise token replaced")
    parser.add_argument("--p_mr", default=0.1, type=float,
                        help="percent of words in each sentence to be mention replaced")
    parser.add_argument("--p_sis", default=0.1, type=float,
                        help="percent of words in each sentence to be shuffle")
    parser.add_argument("--p_sr", default=0.1, type=float,
                        help="percent of words in each sentence to be replaced by synonyms")
    args = parser.parse_args()

    main(args.input, args.output, args.p_power, args.p_lwtr, args.p_mr, args.p_sis, args.p_sr, args.num_aug)
